@@ -16,19 +16,36 @@ sub environment {
     %environment = %args;
 }
 
+my %lib = map { $_ => 1 } @Config{qw(sitearchexp archlibexp)};
+
+sub _insert_before_sitelib {
+    my($inc) = @_;
+
+    my $index;
+    for my $i (0..$#INC) {
+        $index = $i, last if $lib{$INC[$i]};
+    }
+
+    if ($index) {
+        splice @INC, $index, 0, $inc;
+    } else {
+        warn "Can't find \@INC entry for $Config{sitearchexp}";
+    }
+}
+
 sub bootstrap {
     my $class = shift;
+    _insert_before_sitelib(Carmel::Runtime::Guard->new);
     unshift @INC,
       Carmel::Runtime::FastINC->new($class->modules),
-      $class->sharedir,
-      Carmel::Runtime::Guard->new;
+      $class->sharedir;
 }
 
 sub require_all {
     my($class, @phase) = @_;
 
     require Module::Runtime;
-    my $modules = Carmel::Runtime->required_modules(@phase);
+    my $modules = $class->required_modules(@phase);
     while (my($module, $version) = each %$modules) {
         next if $module eq 'perl';
         Module::Runtime::use_module($module, $version);
